package com.gic.message.service.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthModeEnum;
import com.gic.auth.dto.UnionEnterpriseResourceDTO;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.binlog.base.entity.GicField;
import com.gic.binlog.base.entity.GicRecord;
import com.gic.binlog.base.entity.enums.GicRecordType;
import com.gic.message.service.MessageHandler;
import com.gic.message.utils.ListToMapUtil;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreAuthorizationApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

@Component
public class StoreCustomMessageHandler implements MessageHandler {
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreIndexRefreshHandler storeIndexRefreshHandler;
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    @Override
    public void handler(ConsumerRecord<String, GicRecord> consumerRecord) {
        GicRecord value = consumerRecord.value();
        Map<String, GicField> fieldMap = ListToMapUtil.listToMap(consumerRecord);
        Integer enterpriseId = 0, storeInfoId = 0, storeFieldId = 0;
        if(GicRecordType.INSERT.value() == value.getRecordType().value() ||
                GicRecordType.DELETE.value() == value.getRecordType().value()){
            enterpriseId = Integer.valueOf(fieldMap.get("enterprise_id").getValue());
            storeFieldId = Integer.valueOf(fieldMap.get("store_field_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("store_info_id").getValue());
            this.dealCustomChange(enterpriseId, storeFieldId, storeInfoId, fieldMap);
        }
        if(GicRecordType.UPDATE.value() == value.getRecordType().value()){
            enterpriseId = Integer.valueOf(fieldMap.get("new_enterprise_id").getValue());
            storeFieldId = Integer.valueOf(fieldMap.get("new_store_field_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("new_store_info_id").getValue());
            if(!fieldMap.get("old_value").getValue().equals(fieldMap.get("new_value").getValue())){
                this.dealCustomChange(enterpriseId, storeFieldId, storeInfoId, fieldMap);
            }
        }
    }

    private void dealCustomChange(Integer enterpriseId, Integer storeFieldId, Integer storeInfoId, Map<String, GicField> fieldMap){
        String key = "enterprise:store:storeField:" + enterpriseId + ":" + storeFieldId;
        Object existObj = RedisUtil.getCache(key);
        if(existObj != null){
            ServiceResponse<List<StoreDTO>> listStoreResponse = this.storeApiService.listStoreByStoreInfoId(storeInfoId);
            List<StoreDTO> listStore = listStoreResponse.getResult();
            for(StoreDTO storeDTO : listStore){
                if(StoreOwnTypeEnum.OWNER.getCode() == storeDTO.getOwnType()){
                    this.storeIndexRefreshHandler.refreshStoreIndex(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
                }
            }
            this.dealStoreInfoChange(enterpriseId, storeInfoId, fieldMap);
            ServiceResponse<List<StoreDTO>> listStoreResponse1 = this.storeApiService.listStoreByStoreInfoId(storeInfoId);
            List<StoreDTO> listStore1 = listStoreResponse1.getResult();
            for(StoreDTO storeDTO : listStore1){
                this.storeIndexRefreshHandler.refreshStoreIndex(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
            }
        }
    }

    private void dealStoreInfoChange(Integer enterpriseId, Integer storeInfoId, Map<String, GicField> fieldMap){
        ServiceResponse<List<UnionEnterpriseResourceDTO>> resourceResponse = this.unionEnterpriseApiService.listStoreResourceByEnterpriseId(enterpriseId);
        if(CollectionUtils.isNotEmpty(resourceResponse.getResult())){
            for(UnionEnterpriseResourceDTO dto : resourceResponse.getResult()){
                ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService.getStoreWidget(dto.getResource().intValue());
                if(!storeWidget.isSuccess()){
                    continue;
                }
                StoreWidgetDTO storeWidgetDTO = storeWidget.getResult();
                if(storeWidgetDTO != null){
                    if(AuthModeEnum.YES.getCode() == storeWidgetDTO.getAuthMode()){
                        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
                        storeSearchDTO.setStoreInfoIds(String.valueOf(storeInfoId));
                        storeSearchDTO.setSearchJson(storeWidgetDTO.getSearchParam());
                        storeSearchDTO.setEnterpriseId(enterpriseId);
                        ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStore(storeSearchDTO, 1, 1);
                        if(!response.isSuccess()){
                            continue;
                        }
                        if(CollectionUtils.isNotEmpty(response.getResult().getResult())){
                            StoreDTO storeDTO = new StoreDTO();
                            storeDTO.setStoreInfoId(storeInfoId);
                            storeDTO.setEnterpriseId(enterpriseId);
                            storeDTO.setFromEnterpriseId(dto.getUnionEnterpriseId());
                            this.storeAuthorizationApiService.saveStore(storeDTO);
                        }else {
                            this.storeAuthorizationApiService.cancelStore(storeInfoId, dto.getUnionEnterpriseId());
                        }
                    }
                }
            }
        }
    }
}
