package com.gic.message.service.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthModeEnum;
import com.gic.auth.dto.UnionEnterpriseResourceDTO;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.binlog.base.entity.GicField;
import com.gic.binlog.base.entity.GicRecord;
import com.gic.binlog.base.entity.enums.GicRecordType;
import com.gic.message.service.MessageHandler;
import com.gic.message.utils.ListToMapUtil;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreAuthorizationApiService;
import com.gic.store.service.StoreRegionApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

@Component
public class StoreInfoMessageHandler implements MessageHandler {
    @Autowired
    private StoreIndexRefreshHandler storeIndexRefreshHandler;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;
    @Autowired
    private StoreRegionApiService storeRegionApiService;

    @Override
    public void handler(ConsumerRecord<String, GicRecord> consumerRecord) {
        GicRecord value = consumerRecord.value();
        Map<String, GicField> fieldMap = ListToMapUtil.listToMap(consumerRecord);
        Integer enterpriseId = 0, storeInfoId = 0, storeId = 0;
        if(GicRecordType.INSERT.value() == value.getRecordType().value()){

        }
        if(GicRecordType.UPDATE.value() == value.getRecordType().value()){
            enterpriseId = Integer.valueOf(fieldMap.get("new_enterprise_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("new_store_info_id").getValue());
            ServiceResponse<List<StoreDTO>> listStoreResponse = this.storeApiService.listStoreByStoreInfoId(storeInfoId);
            List<StoreDTO> listStore = listStoreResponse.getResult();
            for(StoreDTO storeDTO : listStore){
                if(storeDTO.getOwnType() == StoreOwnTypeEnum.OWNER.getCode()){
                    if(Integer.valueOf(fieldMap.get("old_status").getValue()) == 1 && Integer.valueOf(fieldMap.get("new_status").getValue()) == 0){
                        this.storeApiService.deleteStoreFromES(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
                    }else {
                        this.storeIndexRefreshHandler.refreshStoreIndex(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
                    }
                }
            }
            this.dealStoreInfoChange(enterpriseId, storeInfoId, fieldMap);
            ServiceResponse<List<StoreDTO>> listStoreResponse1 = this.storeApiService.listStoreByStoreInfoId(storeInfoId);
            List<StoreDTO> listStore1 = listStoreResponse1.getResult();
            for(StoreDTO storeDTO : listStore1){
                this.storeIndexRefreshHandler.refreshStoreIndex(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
            }
        }

        if(GicRecordType.DELETE.value() == value.getRecordType().value()){
            storeInfoId = Integer.valueOf(fieldMap.get("store_info_id").getValue());
            ServiceResponse<List<StoreDTO>> listStoreResponse = this.storeApiService.listStoreByStoreInfoId(storeInfoId);
            List<StoreDTO> listStore = listStoreResponse.getResult();
            for(StoreDTO storeDTO : listStore){
               this.storeApiService.deleteStoreFromES(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
            }
        }
    }

    private void dealStoreInfoChange(Integer enterpriseId, Integer storeInfoId, Map<String, GicField> fieldMap){
        ServiceResponse<List<UnionEnterpriseResourceDTO>> resourceResponse = this.unionEnterpriseApiService.listStoreResourceByEnterpriseId(enterpriseId);
        if(CollectionUtils.isNotEmpty(resourceResponse.getResult())){
            for(UnionEnterpriseResourceDTO dto : resourceResponse.getResult()){
                ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService.getStoreWidget(dto.getResource().intValue());
                if(!storeWidget.isSuccess()){
                    continue;
                }
                StoreWidgetDTO storeWidgetDTO = storeWidget.getResult();
                if(storeWidgetDTO != null){
                    if(AuthModeEnum.YES.getCode() == storeWidgetDTO.getAuthMode()){
                        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
                        storeSearchDTO.setStoreInfoIds(String.valueOf(storeInfoId));
                        storeSearchDTO.setSearchJson(storeWidgetDTO.getSearchParam());
                        storeSearchDTO.setEnterpriseId(enterpriseId);
                        ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStore(storeSearchDTO, 1, 1);
                        if(!response.isSuccess()){
                            continue;
                        }
                        if(CollectionUtils.isNotEmpty(response.getResult().getResult())){
                            StoreDTO storeDTO = new StoreDTO();
                            storeDTO.setStoreInfoId(storeInfoId);
                            storeDTO.setEnterpriseId(enterpriseId);
                            storeDTO.setFromEnterpriseId(dto.getUnionEnterpriseId());
                            this.storeAuthorizationApiService.saveStore(storeDTO);
                        }else {
                            this.storeAuthorizationApiService.cancelStore(storeInfoId, dto.getUnionEnterpriseId());
                        }
                        if(Integer.valueOf(fieldMap.get("old_region_id").getValue()) != Integer.valueOf(fieldMap.get("new_region_id").getValue())){
                            StoreSearchDTO storeSearchDTO1 = new StoreSearchDTO();
                            storeSearchDTO1.setRegionId(Integer.valueOf(fieldMap.get("old_region_id").getValue()));
                            storeSearchDTO1.setSearchJson(storeWidgetDTO.getSearchParam());
                            storeSearchDTO1.setEnterpriseId(enterpriseId);
                            ServiceResponse<Page<StoreDTO>> response1 = this.storeApiService.listStore(storeSearchDTO, 1, 1);
                            if(CollectionUtils.isEmpty(response1.getResult().getResult())){
                                this.storeRegionApiService.unAuthorizeRegion(dto.getUnionEnterpriseId(), Integer.valueOf(fieldMap.get("old_region_id").getValue()));
                            }
                        }
                    }
                }
            }
        }
    }
}
